/******************************************************************************
 * CommandDispatcherImpl.java - Dispatch commands through the system
 * $Id: CommandDispatcherImpl.java,v 1.21 2009/02/14 13:08:01 dick Exp $
 * 
 * BuckoFIBS - Backgammon by BuckoSoft
 * Copyright 2008 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: CommandDispatcherImpl.java,v $
 * Revision 1.21  2009/02/14 13:08:01  dick
 * BuckoFIBS is released under the GNU license.
 * Get rid of unused Mode.
 *
 * Revision 1.20  2009/02/14 12:19:54  dick
 * Add SHOW_ABOUT_DIALOG
 *
 * Revision 1.19  2009/02/12 06:25:48  dick
 * Add BEAR_OFF and PLAY_CUE.
 *
 * Revision 1.18  2009/02/11 09:00:43  dick
 * Better beginning and ending your turn.
 * Send a 'who PL' if we get invited by an unknown player.
 *
 * Revision 1.17  2009/02/09 22:45:00  dick
 * SEND_RESIGN to fibs.
 *
 * Revision 1.16  2009/02/06 07:56:48  dick
 * Working on doubles.
 *
 * Revision 1.15  2009/02/03 09:45:59  dick
 * Add some audio events.
 * Setup ClientReceiveParser a little differently.
 *
 * Revision 1.14  2009/02/02 08:41:37  dick
 * send a "board" command on match start, in case it was a resume.
 *
 * Revision 1.13  2009/02/01 21:24:34  dick
 * Add onSavedMatch() and onMissManners() to track other players' behavior.
 *
 * Revision 1.12  2009/01/31 08:47:04  dick
 * YourMove goes to the GameManager now.
 * Handle Undo and Greedy buttons.
 *
 * Revision 1.11  2009/01/31 06:12:42  dick
 * Handle dispatches for the GameManager.
 * Handle button presses from the game toolbar.
 *
 * Revision 1.10  2009/01/28 19:11:42  dick
 * FIBS' watch command doesn't take a parameter.
 *
 * Revision 1.9  2009/01/28 08:34:48  dick
 * Empty the player list and the inviter list when we disconnect from FIBS.
 *
 * Revision 1.8  2009/01/27 19:15:01  dick
 * Add onInviteWarning().
 *
 * Revision 1.7  2009/01/27 06:57:25  dick
 * Handle the system uninviting us.
 *
 * Revision 1.6  2009/01/26 17:32:34  dick
 * Set the clientConnection before setting the parser's mode.
 *
 * Revision 1.5  2009/01/24 17:05:40  dick
 * Don't call yourMove when parsing the board here.
 *
 * Revision 1.4  2009/01/22 05:02:20  dick
 * SEND_CHAT_COMMAND becomes the more generic SEND_COMMAND.
 * onYourMove() doesn't need an argument.
 *
 * Revision 1.3  2009/01/18 06:38:25  dick
 * RECEIVE_NETWORK_MESSAGE is obsolete.
 *
 * Revision 1.2  2009/01/18 06:18:09  dick
 * State is private.
 *
 * Revision 1.1  2009/01/18 04:52:32  dick
 * CommandDispatcher becomes CommandDispatcherImpl.
 * CommandDispatcher is now the interface describing the API for the dispatcher.
 *
 * Revision 1.28  2009/01/16 07:28:00  dick
 * Send a chat/command to the fibs server.
 *
 * Revision 1.27  2009/01/13 18:27:39  dick
 * Handle Ready To Play
 *
 * Revision 1.26  2009/01/12 20:09:33  dick
 * Set the FibsAttributes in the ClientConnection when initializing him.
 *
 * Revision 1.25  2009/01/10 05:04:17  dick
 * Handle MATCH_OVER.
 *
 * Revision 1.24  2009/01/09 06:58:30  dick
 * Chat message handling.
 * ShowPrefs4Msgs dialog.
 *
 * Revision 1.23  2009/01/08 01:35:00  dick
 * Update the entire board after parsing the fibs board string.
 *
 * Revision 1.22  2009/01/07 02:35:05  dick
 * Generalize building a connection (commonize Connect vs. Register).
 *
 * Revision 1.21  2009/01/06 08:05:28  dick
 * Handle registering a new user.
 *
 * Revision 1.20  2009/01/05 07:09:31  dick
 * ConnectToServerDialog and CreateAccountDialog moved to account.
 *
 * Revision 1.19  2009/01/04 19:57:26  dick
 * Add the create new account dialog.
 *
 * Revision 1.18  2008/12/20 23:02:10  dick
 * Support for Own Info.
 *
 * Revision 1.17  2008/12/17 04:37:30  dick
 * Cleanup Network Disconnect.
 *
 * Revision 1.16  2008/12/15 01:49:21  dick
 * PlayerChanged goes to the list pane, not the model, so we update the count.
 *
 * Revision 1.15  2008/12/14 06:43:55  dick
 * Watching switches to the boardpane.
 *
 * Revision 1.14  2008/12/13 06:57:11  dick
 * Move ClientConnection, CookieMonster, and FIBSMessages to their own network package.
 *
 * Revision 1.13  2008/12/13 06:22:07  dick
 * Handle NETWORK_CONNECT and NETWORK_DISCONNECT which twiddle some gui objects.
 *
 * Revision 1.12  2008/12/12 23:23:48  dick
 * If writeSystemMessage is an error message, then make the system tab visible.
 *
 * Revision 1.11  2008/12/12 14:45:02  dick
 * Add the WATCH handler.
 *
 * Revision 1.10  2008/12/11 20:28:21  dick
 * Add AcceptInvitation, DeclineInvitation, StartGame, and Board.
 *
 * Revision 1.9  2008/12/11 09:59:24  dick
 * Display all network received messages if user enabled.
 *
 * Revision 1.8  2008/12/11 08:47:09  dick
 * Add showPreferencesDialog().
 * Add inbound chat message handling.
 *
 * Revision 1.7  2008/12/10 18:01:58  dick
 * Scroll the text pane to the bottom after outputting a message.
 * Display system messages in color.
 *
 * Revision 1.6  2008/12/09 19:37:25  dick
 * Handle being invited to a match.
 *
 * Revision 1.5  2008/12/09 01:49:46  dick
 * writeNetworkMessage() allows us to send data back to the server.
 *
 * Revision 1.4  2008/04/09 20:57:07  dick
 * Handle inviting a player to a match.
 *
 * Revision 1.3  2008/04/01 04:21:53  dick
 * Handle player logging out.
 *
 * Revision 1.2  2008/03/31 07:09:42  dick
 * Dispatching network messages.
 *
 * Revision 1.1  2008/03/30 05:40:50  dick
 * Main command dispatcher for BuckoFIBS.
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is BuckoFIBS, <http://www.buckosoft.com/BuckoFIBS/>.
 * The Initial Developer of the Original Code is Dick Balaska and BuckoSoft, Corp.
 * 
 */
package com.buckosoft.fibs.BuckoFIBS;

import com.buckosoft.fibs.BuckoFIBS.AudioManager.Cue;
import com.buckosoft.fibs.BuckoFIBS.ClientReceiveParser.Mode;
import com.buckosoft.fibs.BuckoFIBS.gui.AboutDialog;
import com.buckosoft.fibs.BuckoFIBS.gui.MainDialog;
import com.buckosoft.fibs.BuckoFIBS.gui.PreferencesDialog;
import com.buckosoft.fibs.BuckoFIBS.gui.PreferencesForMessagesDialog;
import com.buckosoft.fibs.BuckoFIBS.gui.SystemMessagesTextPane;
import com.buckosoft.fibs.BuckoFIBS.gui.account.ConnectToServerDialog;
import com.buckosoft.fibs.BuckoFIBS.gui.account.CreateAccountDialog;
import com.buckosoft.fibs.BuckoFIBS.gui.account.CreateAccountStatusInterface;
import com.buckosoft.fibs.domain.Line;
import com.buckosoft.fibs.domain.Player;
import com.buckosoft.fibs.domain.SavedMatch;
import com.buckosoft.fibs.net.ClientConnection;
import com.buckosoft.fibs.net.FIBSMessages;

/** Dispatch commands through the system.
 * @author Dick Balaska
 * @since 2008/03/30
 * @version $Revision: 1.21 $ <br> $Date: 2009/02/14 13:08:01 $
 * @see <a href="http://cvs.buckosoft.com/Projects/BuckoFIBS/BuckoFIBS/src/com/buckosoft/fibs/BuckoFIBS/CommandDispatcherImpl.java">cvs CommandDispatcherImpl.java</a>
 */
public class CommandDispatcherImpl implements CommandDispatcher, FIBSMessages {
	private	final static boolean DEBUG = false;

	private	MainDialog		mainDialog;
	private	BFProperties	properties;
	private	ClientConnection	clientConnection = null;

	private CreateAccountStatusInterface createAccountStatusInterface = null;

	private	final static String eol = "\r\n";

	/** Set the reference to the instance of MainDialog
	 * @param mainDialog The MainDialog that is running
	 */
	public	void setMainDialog(MainDialog mainDialog) {
		this.mainDialog = mainDialog;
	}
	/** Set the Properties/User preferences
	 * @param properties The loaded properties
	 */
	public	void setProperties(BFProperties properties) {
		this.properties = properties;
	}

	/** Get a reference to our properties
	 * @return The properties object
	 */
	public	BFProperties	getProperties() {
		return(this.properties);
	}

	/** Primary entry point to the command dispatcher
	 * @param command The command to execute
	 */
	public void dispatch(Command command) {
		switch (command) {
		case SHOW_CONNECTION_DIALOG:
			onShowConnectionDialog();
			break;
		case SHOW_NEW_ACCOUNT_DIALOG:
			onShowNewAccountDialog();
			break;
		case DISCONNECT_FROM_NETWORK:
			onNetworkDisconnect();
			break;
		case CONNECT_TO_SERVER:
			onConnectToServer();
			break;
		case SHOW_PREFERENCES_DIALOG:
			onShowPreferencesDialog();
			break;
		case SHOW_PREFS4MSGS_DIALOG:
			onShowPrefs4MsgsDialog();
			break;
		case SHOW_ABOUT_DIALOG:
			onShowAboutDialog();
			break;
		case START_GAME:
			onStartGame();
			break;
		case SHUTTING_DOWN:
			onShutdown();
			break;
		case NETWORK_CONNECTED:
			onNetworkConnected();
			break;
		case WATCHING:
			onWatching();
			break;
		case ROLL_OR_DOUBLE:
			onRollOrDouble();
			break;
		case ACCEPT_OR_DECLINE_DOUBLE:
			onAcceptOrDeclineDouble();
			break;
		case TOGGLE_READY_TO_PLAY:
			onToggleReadyToPlay();
			break;
		case BEAR_OFF:
			this.mainDialog.getBoard().clearDice();
			this.mainDialog.playSound(Cue.PickUpDice);
			this.mainDialog.youCantMove();		// you can't move because your turn is over
			break;
		case SEND_ROLL:
			this.writeNetworkMessageln("roll");		
			this.mainDialog.getBoard().setYourTurnToRollOrDouble(false);
			this.mainDialog.updateBoard();
			break;
		case SEND_DOUBLE:
			this.writeNetworkMessageln("double");		
			this.mainDialog.getBoard().setYourTurnToRollOrDouble(false);
			this.mainDialog.updateBoard();
			break;
		case SEND_ACCEPT:
			this.writeNetworkMessageln("accept");		
			break;
		case SEND_REJECT:
			this.writeNetworkMessageln("reject");		
			break;
			
		default:
			writeSystemMessageln("Dispatcher: Unhandled command " + command);
			throw new RuntimeException("Unhandled command " + command);
		}
	}

	/** Primary entry point to the command dispatcher
	 * what takes a single string as an argument
	 * @param command The Command to Execute
	 * @param arg1 A string that the dispatched function wants
	 */
	public void dispatch(Command command, String arg1) {
		switch (command) {
		case SYSTEM_MESSAGE:
			writeSystemMessage(arg1);
			break;
/*		case RECEIVE_NETWORK_MESSAGE:
			if (this.properties.isDisplayRecv())
				writeSystemMessageln(SystemMessagesTextPane.NETWORKIN, arg1);
			this.clientReceiveParser.parseMessage(arg1);
			break;
*/
		case PLAYER_GONE:
			this.mainDialog.getPlayerListPane().playerGone(arg1);
			this.mainDialog.getInviterTableModel().uninvited(arg1);
			break;
		case SAVED_MATCH:
			onSavedMatch(arg1);
			break;
		case MISS_MANNERS:
			onMissManners(arg1);
			break;
		case WATCH:
			onWatch(arg1);
			break;
		case ACCEPT_INVITATION:
			onAcceptInvitation(arg1);
			this.mainDialog.getInviterTableModel().uninvited(arg1);
			break;
		case UNINVITED:
			onUninvited(arg1);
			break;
		case OWN_INFO:
			parseOwnInfo(arg1);
			break;
		case FIBS_BOARD:
			parseFibsBoard(arg1);
			break;
		case DOUBLE:
			break;
		case YOUR_MOVE:
			onYourMove(Integer.parseInt(arg1));
			break;
		case BAD_NEW_USER:
			this.createAccountStatusInterface.setStatus(arg1);
			this.createAccountStatusInterface.onRegisterFailure();
			break;
		case SEND_RESIGN:
			this.writeNetworkMessageln("resign " + arg1);
			break;
		case SEND_MOVE:
			this.mainDialog.getBoard().clearDice();
			this.mainDialog.playSound(Cue.PickUpDice);
			this.mainDialog.updateBoard();
			this.writeNetworkMessageln(arg1);
			this.mainDialog.writeGameMessageln(SystemMessagesTextPane.NETWORKOUT, arg1);
			break;
		case SEND_COMMAND:
			this.writeNetworkMessageln(arg1);		
			break;
		default:
			writeSystemMessageln("Dispatcher: Unhandled command " + command);
			throw new RuntimeException("Unhandled command " + command);
		}
	}

	/** Primary entry point to the command dispatcher
	 * what takes two strings as an argument
	 * @param command The Command to Execute
	 * @param arg1 A string that the dispatched function wants
	 * @param arg2 Another string that the dispatched function wants
	 */
	public void dispatch(Command command, String arg1, String arg2) {
		switch (command) {
		case INVITED:
			onInvited(arg1, arg2);
			break;
		case INVITE:
			onInvite(arg1, arg2);
			break;
		case INVITE_WARNING:
			onInviteWarning(arg1, arg2);
			break;
		case MATCH_OVER:
			this.mainDialog.onMatchOver(arg1, arg2);
			break;
		default:
			writeSystemMessageln("Dispatcher: Unhandled command " + command);
			throw new RuntimeException("Unhandled command" + command);
		}
	}

	/** Primary entry point to the command dispatcher
	 * what takes an Object as an argument
	 * @param command The Command to Execute
	 * @param obj An Object that the dispatched function wants
	 */
	public void dispatch(Command command, Object obj) {
		boolean b;
		switch (command) {
		case GAME_MOVE:
			this.mainDialog.getGameManager().addLine((Line)obj);
			break;
		case PLAYER_CHANGED:
			onPlayerChanged((Player)obj);
			break;
		case REGISTER_NEW_USER:
			createAccountStatusInterface = (CreateAccountStatusInterface)obj;
			onRegisterNewUser();
			break;
		case READY_TO_PLAY:
			b = (Boolean)obj;
			this.mainDialog.setReadyToPlay(b);
			break;
		case TOGGLE_DOUBLE:
			b = (Boolean)obj;
			this.mainDialog.setAskDouble(b);
			break;
		case PLAY_CUE:
			AudioManager.Cue cue = (AudioManager.Cue)obj;
			this.mainDialog.playSound(cue);
			break;
		default:
			writeSystemMessageln("Dispatcher: Unhandled command " + command);
			throw new RuntimeException("Unhandled command" + command);
		}
	}

	/** Write a message to the System message pane in the normal color
	 * @param s The message to write
	 */
	public	void writeSystemMessage(String s) {
		this.mainDialog.getSystemMessagesTextPane().appendMessage(s);
		this.mainDialog.systemMessagesScrollToBottom();
	}
	
	/** Write a message to the System message pane in the normal color.
	 * Terminate the message with a crlf.
	 * @param s The message to write
	 */
	public void writeSystemMessageln(String s) {
		this.mainDialog.getSystemMessagesTextPane().appendMessage(s + eol);		
		this.mainDialog.systemMessagesScrollToBottom();
	}

	/** Write a message to the status widget in the registerUser dialog
	 * @param s The message to write
	 */
	public void writeRegisterUserMessage(String s) {
		if (this.createAccountStatusInterface != null)
			this.createAccountStatusInterface.setStatus(s);
	}

	/** Write a message to the System message pane in the specified color.
	 * @param type The style of the text
	 * @param s The message to write
	 */
	public void writeSystemMessage(int type, String s) {
		this.mainDialog.getSystemMessagesTextPane().appendMessage(type, s);		
		this.mainDialog.systemMessagesScrollToBottom();
	}

	/** Write a message to the System message pane in the specified color.
	 * Terminate the message with a crlf.
	 * @param type The style of the text
	 * @param s The message to write
	 */
	public void writeSystemMessageln(int type, String s) {
		this.mainDialog.getSystemMessagesTextPane().appendMessage(type, s + eol);		
		this.mainDialog.systemMessagesScrollToBottom();
		if (type == SystemMessagesTextPane.ERROR)
			this.mainDialog.setSystemMessagesTabVisible();
	}

	/** Write a message to the chat message pane in the normal color.
	 * Terminate the message with a crlf.
	 * @param name The user what sent the message
	 * @param cookie The mode used to send the message (shout, whisper, kibitz, etc)
	 * @param text The message to write
	 */
	public void writeChatMessageln(String name, int cookie, String text) {
		final String mode[]  = {"says", "shouts", "whispers", "kibitzes",
								"say",  "shout",  "whisper",  "kibitz"};
		int cookieMode = cookie - CLIP_SAYS;
		if (cookie == CLIP_YOU_SAY) {
			String[] ss = text.split(" ", 2);
			this.mainDialog.addChatMessage("You tell " + ss[0] + ": " + ss[1] + eol);
			return;
		}
		this.mainDialog.addChatMessage(name + " " + mode[cookieMode] + ": " + text + eol);		
		//this.mainDialog.systemMessagesScrollToBottom();
	}

	public void writeGameMessageln(String s) {
		this.mainDialog.writeGameMessageln(s);
	}
	
	/** Send a message to the fibs server
	 * @param s The message to send
	 * @return success
	 */
	public boolean writeNetworkMessage(String s) {
		if (this.clientConnection != null) {
			this.clientConnection.sendMessage(s);
			return(true);
		} else 
			return(false);
	}
	
	/** Send a message to the fibs server appending a crlf to the end
	 * @param s The message to send
	 * @return success
	 */
	public boolean writeNetworkMessageln(String s) {
		return(this.writeNetworkMessage(s + eol));
	}

	private	void onShowConnectionDialog() {
		ConnectToServerDialog dialog = new ConnectToServerDialog(mainDialog, this);
		dialog.setVisible(true);
	}

	private	void onShowNewAccountDialog() {
		CreateAccountDialog dialog = new CreateAccountDialog(mainDialog, this);
		dialog.setVisible(true);
	}

	private	void onConnectToServer() {
		if (createAccountStatusInterface != null) {
			createAccountStatusInterface.onRegisterSuccess();
			createAccountStatusInterface = null;
		}
		startClientConnection(Mode.Run);
	}

	private void onRegisterNewUser() {
		int profileId = this.properties.getSelectedProfile();
		String s = "Connecting to "
			+ this.properties.getServerName(profileId)
			+ ":"
			+ this.properties.getServerPort(profileId);
		this.createAccountStatusInterface.setStatus(s);
		startClientConnection(Mode.Register);
	}

	private void startClientConnection(Mode mode) {
		if (this.clientConnection != null) {
			if (DEBUG)
				System.out.println("Tear down old ClientConnection");
			this.clientConnection.shutDown();
			this.clientConnection = null;
		}
		if (DEBUG)
			System.out.println("Create new ClientConnection");
		this.clientConnection = new ClientConnection();
		ClientReceiveParser clientReceiveParser = this.mainDialog.getClientReceiveParser();
		this.clientConnection.setClientAdapter(clientReceiveParser);
		this.clientConnection.setFibsAttributes(properties);
		clientReceiveParser.setClientConnection(this.clientConnection);
		clientReceiveParser.setMode(mode);
		this.clientConnection.start();
	}

	private void onShowPreferencesDialog() {
		PreferencesDialog dialog = new PreferencesDialog(mainDialog, this);
		dialog.setVisible(true);
	}

	private void onShowAboutDialog() {
		AboutDialog aboutDialog = new AboutDialog(mainDialog);
		aboutDialog.setVersion(Version.version);
		aboutDialog.setVisible(true);
	}
	private void onShowPrefs4MsgsDialog() {
		PreferencesForMessagesDialog dialog = new PreferencesForMessagesDialog(mainDialog, this);
		dialog.setVisible(true);
	}

	private	void onShutdown() {
		onNetworkDisconnect();
	}

	private void onNetworkConnected() {
		this.writeNetworkMessageln("set boardstyle 3");
//		if (this.properties.isDEBUG_NotReady())
//			this.clientConnection.sendMessage("toggle ready" + eol);
		this.mainDialog.setConnected(true);
	}

	private	void onNetworkDisconnect() {
		if (this.clientConnection != null) {
			this.clientConnection.shutDown();
			this.clientConnection = null;
		}
		this.mainDialog.setConnected(false);
		this.mainDialog.getInviterTableModel().removeAll();
		this.mainDialog.getPlayerTableModel().removeAll();
	}

	private void onPlayerChanged(Player p) {
		p.setSavedMatch(this.mainDialog.getBoard().getSavedMatchString(p.getName()));
		p.setMissManners(this.mainDialog.getBoard().getMissMannersString(p.getName()));
		this.mainDialog.getPlayerListPane().playerChanged(p);
		if (p.getOpponent().length() > 0)	// If he is playing, then any invite of his is dead.
			onUninvited(p.getName());
		this.mainDialog.getInviterTableModel().playerChanged(p);
		if (p.getName().equals(this.properties.getUserName())) {
			this.mainDialog.getBoard().setReady(p.isReady());
			this.mainDialog.redrawOwnInfo();
		}
	}

	private	void onInvite(String playerName, String length) {
		this.writeNetworkMessageln("invite " + playerName + " " + length);
	}

	private	void onSavedMatch(String s) {
		SavedMatch sm = this.mainDialog.getBoard().parseSavedMatch(s);
		Player p = this.mainDialog.getPlayerTableModel().getPlayer(sm.getName());
		if (p != null) {
			p.setSavedMatch(this.mainDialog.getBoard().getSavedMatchString(p.getName()));
			this.mainDialog.getPlayerListPane().playerChanged(p);
		}
	}

	private	void onMissManners(String s) {
		SavedMatch sm = this.mainDialog.getBoard().parseMissManners(s);
		if (sm == null)
			return;
		Player p = this.mainDialog.getPlayerTableModel().getPlayer(sm.getName());
		if (p != null) {
			p.setMissManners(this.mainDialog.getBoard().getSavedMatchString(p.getName()));
			this.mainDialog.getPlayerListPane().playerChanged(p);
		}
	}

	private	void onWatch(String playerName) {
		this.writeNetworkMessageln("watch " + playerName);
	}

	private	void onWatching() {
		this.mainDialog.setBoardTabVisible();
		this.mainDialog.getGameManager().reset();
		this.writeNetworkMessageln("board");
	}

	private	void onAcceptInvitation(String playerName) {
		this.writeNetworkMessageln("join " + playerName);
	}

	private	void onRollOrDouble() {
		this.mainDialog.getBoard().setYourTurnToRollOrDouble(true);
		this.mainDialog.updateBoard();
	}

	private void onAcceptOrDeclineDouble() {
		this.mainDialog.getBoard().setAcceptDeclineDouble();
		this.mainDialog.updateBoard();
	}

	private void onToggleReadyToPlay() {
		this.writeNetworkMessageln("toggle ready");
	}

	private void onInvited(String playerName, String matchLength) {
		Player p = this.mainDialog.getPlayerTableModel().getPlayer(playerName);
		if (p == null) {
			this.writeSystemMessageln(SystemMessagesTextPane.ERROR, 
					"Unknown player '" + playerName + "' tried to invite you");
			p = new Player();
			p.setName(playerName);
			this.dispatch(CommandDispatcher.Command.PLAYER_CHANGED, p);
			this.writeNetworkMessageln("who " + playerName);
		}
		this.mainDialog.playSound(Cue.Invited);
		this.mainDialog.getInviterTableModel().invited(p, matchLength);
		this.mainDialog.setInvitedTabVisible();
	}

	private	void onInviteWarning(String playerName, String warning) {
		this.mainDialog.getInviterTableModel().setWarning(playerName, warning);		
	}

	private void onUninvited(String playerName) {
		this.mainDialog.getInviterTableModel().uninvited(playerName);
	}

	private void onStartGame() {
		this.mainDialog.getGameManager().reset();
		this.mainDialog.getBoard().resetGame();
		this.mainDialog.redrawOwnInfo();
		this.mainDialog.setBoardTabVisible();
		this.writeNetworkMessageln("board");
	}
	
	private void onYourMove(int diceToMove) {
		this.mainDialog.getBoard().setYourTurnToRollOrDouble(false);
		this.mainDialog.getGameManager().yourMove(diceToMove);
		this.mainDialog.playSound(Cue.YourTurn);
	}

	private	void parseFibsBoard(String board) {
		this.mainDialog.getBoard().parseFibsBoard(board);
		this.mainDialog.updateBoard();
//		if (this.mainDialog.getBoard().getWhosTurn() == -1)
//			this.mainDialog.yourMove();
	}

	private void parseOwnInfo(String s) {
		this.mainDialog.getBoard().parseOwnInfo(s);
		if (this.properties.isFibsReadyToPlay() != this.mainDialog.getBoard().isReady())
			this.writeNetworkMessageln("toggle ready");
		else
			this.mainDialog.setReadyToPlay(this.properties.isFibsReadyToPlay());
		this.mainDialog.redrawOwnInfo();
	}

	/* (non-Javadoc)
	 * @see com.buckosoft.fibs.BuckoFIBS.gui.board.ToolbarHandler#buttonPressed(com.buckosoft.fibs.BuckoFIBS.gui.board.ToolbarHandler.Button)
	 */
	@Override
	public void buttonPressed(Button button) {
		if (button == Button.First)
			this.mainDialog.getGameManager().goFirst();
		else if (button == Button.Prev)
			this.mainDialog.getGameManager().goPrev();
		else if (button == Button.Next)
			this.mainDialog.getGameManager().goNext();
		else if (button == Button.Last)
			this.mainDialog.getGameManager().goLast();
		else if (button == Button.Undo)
			this.mainDialog.getGameManager().goUndo();
		else if (button == Button.Greedy)
			this.writeNetworkMessageln("toggle greedy");
		else if (button == Button.AskDoubles)
			this.writeNetworkMessageln("toggle double");
		else if (button == Button.Resign)
			this.mainDialog.showResignOutDialog();
	}
}
