/******************************************************************************
 * MainDialog.java - Load and manage all of the gui components in BuckoFIBS
 * $Id: MainDialog.java,v 1.38 2009/02/14 12:21:17 dick Exp $
 * 
 * BuckoFIBS - Backgammon by BuckoSoft
 * Copyright 2008 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: MainDialog.java,v $
 * Revision 1.38  2009/02/14 12:21:17  dick
 * Display the version in the startup message.
 *
 * Revision 1.37  2009/02/12 06:26:18  dick
 * Only make noise if playCues property is set.
 *
 * Revision 1.36  2009/02/11 09:02:35  dick
 * Gracefully continue if the AudioManager fails to initialize.
 * Write "move bar-24" in color to the game window.
 *
 * Revision 1.35  2009/02/09 22:45:40  dick
 * shutdown() cancels pending timers.
 * Handle resign (both ways).
 *
 * Revision 1.34  2009/02/06 07:56:48  dick
 * Working on doubles.
 *
 * Revision 1.33  2009/02/03 09:46:22  dick
 * MainDialog controls the ClientReceiveParser now.
 *
 * Revision 1.32  2009/02/01 21:25:08  dick
 * redrawOwnInfo() includes drawing our ready state.
 *
 * Revision 1.31  2009/01/31 08:48:48  dick
 * GameManager needs a reference to MainDialog.
 *
 * Revision 1.30  2009/01/28 22:32:35  dick
 * Setup the GameManager and the ReplayToolbar.
 *
 * Revision 1.29  2009/01/28 19:40:14  dick
 * package com.buckosoft.fibs.gui.* becomes com.buckosoft.fibs.BuckoFIBS.gui.*.
 *
 * Revision 1.28  2009/01/22 05:03:27  dick
 * yourMove() doesn't need an argument.
 *
 * Revision 1.27  2009/01/18 04:54:33  dick
 * The constructor now needs the CommandDispatcher sent to it.
 * Add yourMove(b) which turns on and off the player moving his checkers.
 *
 * Revision 1.26  2009/01/16 07:27:34  dick
 * ChatPane needs the CommandDispatcher now.
 *
 * Revision 1.25  2009/01/13 18:27:39  dick
 * Handle Ready To Play
 *
 * Revision 1.24  2009/01/10 05:05:14  dick
 * Handle onMatchOver.
 * Wrap scrolling with a try block.  Sometimes scrolling throws a wacky exception.  Why?
 *
 * Revision 1.23  2009/01/09 06:58:56  dick
 * GameMessages panel.
 *
 * Revision 1.22  2009/01/08 18:32:29  dick
 * Move all of the Board drawing/handling to BoardTab.
 *
 * Revision 1.21  2009/01/08 01:34:34  dick
 * BoardPane gets subsumed under BoardPanel so that we can also have a MatchStatusPanel.
 *
 * Revision 1.20  2008/12/20 23:03:27  dick
 * Add redrawOwnInfo.
 *
 * Revision 1.19  2008/12/15 01:50:11  dick
 * getPlayerListPane is public.
 *
 * Revision 1.18  2008/12/13 06:22:49  dick
 * Pass along the setConnected() event to the toolbar.
 *
 * Revision 1.17  2008/12/12 23:24:08  dick
 * If writeSystemMessage is an error message, then make the system tab visible.
 *
 * Revision 1.16  2008/12/11 20:28:53  dick
 * Make the board visible on StartNewGame.
 *
 * Revision 1.15  2008/12/11 08:48:48  dick
 * Handle autoConnect.
 * Add inbound chat handling.
 *
 * Revision 1.14  2008/12/10 18:05:53  dick
 * Add systemMessagesScrollToBottom()
 *
 * Revision 1.13  2008/12/09 19:38:13  dick
 * Add setInvitedTabVisible() which displays the invitations when a new one comes in.
 *
 * Revision 1.12  2008/12/09 01:52:02  dick
 * Add the InviterListPane.
 *
 * Revision 1.11  2008/12/08 10:16:48  dick
 * Initialize the board to the start positions.
 *
 * Revision 1.10  2008/12/08 06:45:02  dick
 * Create a Board and attach it to the BoardPane.
 *
 * Revision 1.9  2008/12/07 22:51:40  dick
 * Javadoc.
 *
 * Revision 1.8  2008/10/08 23:10:03  dick
 * Declare and use the chatPane.
 *
 * Revision 1.7  2008/04/09 20:57:38  dick
 * playerListPane needs the commandDispatcher.
 *
 * Revision 1.6  2008/04/05 01:43:06  dick
 * Add the game board to the top left panel.
 *
 * Revision 1.5  2008/04/02 23:24:43  dick
 * Move playerList to it's own package.
 *
 * Revision 1.4  2008/04/01 03:46:40  dick
 * Get rid of the builtin PlayerListPane and use the external one.
 *
 * Revision 1.3  2008/03/31 07:10:35  dick
 * Expose the playerTableModel.
 *
 * Revision 1.2  2008/03/30 05:42:58  dick
 * More skeleton work.  MainDialog is fleshing out...
 *
 * Revision 1.1  2008/03/29 08:57:43  dick
 * BuckoFIBS skeleton checkin.
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is BuckoFIBS, <http://www.buckosoft.com/BuckoFIBS/>.
 * The Initial Developer of the Original Code is Dick Balaska and BuckoSoft, Corp.
 * 
 */
package com.buckosoft.fibs.BuckoFIBS.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Rectangle;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;

import com.buckosoft.fibs.BuckoFIBS.AudioManager;
import com.buckosoft.fibs.BuckoFIBS.BFProperties;
import com.buckosoft.fibs.BuckoFIBS.ClientReceiveParser;
import com.buckosoft.fibs.BuckoFIBS.CommandDispatcher;
import com.buckosoft.fibs.BuckoFIBS.GameManager;
import com.buckosoft.fibs.BuckoFIBS.Version;
import com.buckosoft.fibs.BuckoFIBS.gui.board.BoardTab;
import com.buckosoft.fibs.BuckoFIBS.gui.board.ReplayToolbar;
import com.buckosoft.fibs.BuckoFIBS.gui.board.ResignOut;
import com.buckosoft.fibs.BuckoFIBS.gui.chatWindow.ChatPane;
import com.buckosoft.fibs.BuckoFIBS.gui.inviterList.InviterListPane;
import com.buckosoft.fibs.BuckoFIBS.gui.inviterList.InviterTableModel;
import com.buckosoft.fibs.BuckoFIBS.gui.playerList.PlayerListPane;
import com.buckosoft.fibs.BuckoFIBS.gui.playerList.PlayerTableModel;
import com.buckosoft.fibs.board.Board;

/** Manage all of the gui components in BuckoFIBS
 * @author Dick Balaska
 * @since 2008/03/29
 * @version $Revision: 1.38 $ <br> $Date: 2009/02/14 12:21:17 $
 * @see <a href="http://cvs.buckosoft.com/Projects/BuckoFIBS/BuckoFIBS/src/com/buckosoft/fibs/BuckoFIBS/gui/MainDialog.java">cvs MainDialog.java</a>
 */
public class MainDialog extends JFrame {
	private	final static boolean DEBUG = false;
	private static final long serialVersionUID = 1L;
	private JPanel			jContentPane = null;
	private ToolBar			toolbar = null;

	private JSplitPane		topBottomSplitPane = null;
	private JSplitPane		topLeftRightSplitPane = null;
	private JSplitPane		bottomLeftRightSplitPane = null;
	private JTabbedPane		topLeftTabbedPane = null;
	private PlayerListPane	playerListPane = null;

	private JTabbedPane			topRightTabbedPane = null;
	private SystemMessagesTextPane	systemMessagesTextPane = null;
	private JScrollPane 		systemMessagesScrollPane = null;
	private Dimension			systemMessagesDimension = null;  //  @jve:decl-index=0:
	private	Rectangle			systemMessagesRectangle = null;  //  @jve:decl-index=0:

	private SystemMessagesTextPane	gameMessagesTextPane = null;
	private JScrollPane 			gameMessagesScrollPane = null;
	private Dimension				gameMessagesDimension = null;  //  @jve:decl-index=0:
	private	Rectangle				gameMessagesRectangle = null;  //  @jve:decl-index=0:
	
	private InviterListPane	inviterListPane = null;
	private	JScrollPane		inviterListScrollPane = null;

	private	ChatPane			bottomRightChatPane = null;  //  @jve:decl-index=0:
	private	BoardTab			boardTab = null;  //  @jve:decl-index=0:visual-constraint="16,1526"
	
	private	BFProperties		properties = null;
	private	CommandDispatcher	commandDispatcher = null;
	private	ClientReceiveParser clientReceiveParser;
	private	GameManager				gameManager;
	private	AudioManager		audioManager;

	private Board			board = new Board();
	private	final String	titleName = "BuckoFIBS";
	private	final static String eol = "\r\n";

	/////////////////////////////////////////////////////////////////////
	/** Create the main dialog
	 * @param properties The Properties to use
	 * @param commandDispatcher The commandDispatcher
	 */
	public MainDialog(BFProperties properties, CommandDispatcher commandDispatcher) {
		super();
		this.properties = properties;
		this.commandDispatcher = commandDispatcher;  //  @jve:decl-index=0:
		this.commandDispatcher.setMainDialog(this);
		this.commandDispatcher.setProperties(this.properties);
		this.clientReceiveParser = new ClientReceiveParser();
		this.clientReceiveParser.setProperties(this.properties);
		this.clientReceiveParser.setCommandDispatcher(this.commandDispatcher);
		this.clientReceiveParser.setMainDialog(this);
		initialize();
		this.gameManager = new GameManager();
		this.gameManager.setMainDialog(this);
		this.audioManager = null;
		try {
			this.audioManager = new AudioManager();
			this.audioManager.initialize(this);
		} catch (Exception ex) {
			System.out.println("Can't init audio manager");
		}
		//this.gameManager.setCommandDispatcher(commandDispatcher);
		this.gameManager.setReplayToolbar(this.getReplayToolbar());
		this.gameManager.reset();
		if (this.properties.isAutoConnect())
			this.commandDispatcher.dispatch(CommandDispatcher.Command.CONNECT_TO_SERVER);
	}

	/** Called when the player is to begin his turn.
	 */ 
	public void yourMove(int diceToMove) {
		this.getBoardTab().yourMove(diceToMove);
	}

	/** Turn off You moving
	 */
	public void youCantMove() {
		this.getBoardTab().getBoardGui().youCantMove();
		this.getBoardTab().updateBoard();
	}

	/** Repaint the backgammon board, after its been manipulated somehow
	 */
	public void updateBoard() {
		this.getBoardTab().updateBoard();
	}

	/** Play a sound
	 * @param cue The sound Cue to play
	 */
	public void playSound(AudioManager.Cue cue) {
		if (this.properties.isAudioCues() && this.audioManager != null)
			this.audioManager.play(cue);
	}

	/** This method initializes this
	 */
	private void initialize() {
		this.board.setStartPositions();
		this.setSize(properties.getMainWidth(), properties.getMainHeight());
		this.setLocation(properties.getMainX(), properties.getMainY());
		this.setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
		this.setTitle(titleName);
		this.setContentPane(getJContentPane());
		this.addWindowListener(new java.awt.event.WindowAdapter() {
			public void windowClosing(java.awt.event.WindowEvent e) {
				mainDialogShuttingDown();
			}
		});
		systemMessagesTextPane.appendMessage("BuckoFIBS-" + Version.version + " Starting up\r\n");
	}

	public PlayerTableModel getPlayerTableModel() {
		return(playerListPane.getPlayerTableModel());
	}
	public InviterTableModel getInviterTableModel() {
		return(this.inviterListPane.getInviterTableModel());
	}
	
	private	ReplayToolbar getReplayToolbar() {
		return(this.boardTab.getJToolBarReplay());
	}

	/** Return a reference to the game manager
	 * @return The GameManager
	 */
	public GameManager getGameManager() {
		return(this.gameManager);
	}

	/** Return a reference to our clientReceiveParser
	 * @return the clientReceiveParser
	 */
	public ClientReceiveParser getClientReceiveParser() {
		return clientReceiveParser;
	}

	/** Make the invited tab visible
	 */
	public void setInvitedTabVisible() {
		this.topRightTabbedPane.setSelectedComponent(this.inviterListScrollPane);
	}

	/** After parsing the "Own info" message from FIBS, update the buttons etc. to match.
	 * 
	 */
	public void redrawOwnInfo() {
		this.setTitle(titleName + " - " + this.board.getName());
		this.setReadyToPlay(this.board.isReady());
		this.getBoardTab().getGameToolBar().setAskDouble(this.board.isAskDouble());

	}

	/** Set the display to show that we are connected to fibs.
	 * @param b true if we're connected
	 */
	public void setConnected(boolean b) {
		this.getToolbar().setConnected(b);
	}

	public void setReadyToPlay(boolean b) {
		this.getToolbar().setReadyToPlay(b);
	}

	public void setAskDouble(boolean b) {
		this.getBoardTab().getGameToolBar().setAskDouble(b);
	}

	public void setSystemMessagesTabVisible() {
		this.topRightTabbedPane.setSelectedComponent(this.systemMessagesScrollPane);
	}

	public void setGameMessagesTabVisible() {
		if (this.gameMessagesScrollPane == null) {
			topRightTabbedPane.addTab("Game", null, getGameMessagesScrollPane(), null);
			this.topRightTabbedPane.setSelectedComponent(this.gameMessagesScrollPane);
		} else {
			this.topRightTabbedPane.setSelectedComponent(this.gameMessagesScrollPane);
		}
	}

	public void setBoardTabVisible() {
		this.topLeftTabbedPane.setSelectedComponent(this.boardTab);
	}

	public void showResignOutDialog() {
		ResignOut ro = new ResignOut(this, this.getBoardTab().getBoardGui(), this.commandDispatcher);
		ro.setVisible(true);
	}

	public void showResignInDialog(String[] ss) {
		int ret = JOptionPane.showConfirmDialog(
					this.getBoardTab().getBoardGui(),
					ss,
					"Opponent wants to resign",
					JOptionPane.OK_CANCEL_OPTION,
					JOptionPane.PLAIN_MESSAGE);
		this.commandDispatcher.writeNetworkMessageln(ret == JOptionPane.OK_OPTION ? "accept" : "reject");
	}

	public void writeGameMessageln(String s) {
		if (this.gameMessagesScrollPane == null) {
			topRightTabbedPane.addTab("Game", null, getGameMessagesScrollPane(), null);
			this.topRightTabbedPane.setSelectedComponent(this.gameMessagesScrollPane);
		}
		this.gameMessagesTextPane.appendMessage(s + eol);
		gameMessagesScrollToBottom();
	}

	/** Write a message to the Game message pane in the specified color.
	 * Terminate the message with a crlf.
	 * @param type The style of the text
	 * @param s The message to write
	 */
	public void writeGameMessageln(int type, String s) {
		this.gameMessagesTextPane.appendMessage(type, s + eol);		
		this.gameMessagesScrollToBottom();
	}

	
	/** Scroll the system messages pane so that the last line is visible.
	 * Use this after adding text to the system messages.
	 */
	public void systemMessagesScrollToBottom() {
		try {
			systemMessagesDimension = systemMessagesTextPane.getSize(systemMessagesDimension);
			systemMessagesRectangle.y = systemMessagesDimension.height;
			systemMessagesScrollPane.getViewport().scrollRectToVisible(systemMessagesRectangle);
		} catch (Exception e) {
			if (DEBUG) {
				System.out.println("Exception ignored");
				e.printStackTrace();
			}
		}
	}

	/** Scroll the system messages pane so that the last line is visible.
	 * Use this after adding text to the system messages.
	 */
	private void gameMessagesScrollToBottom() {
		try {
			gameMessagesDimension = gameMessagesTextPane.getSize(gameMessagesDimension);
			gameMessagesRectangle.y = gameMessagesDimension.height;
			gameMessagesScrollPane.getViewport().scrollRectToVisible(gameMessagesRectangle);
		} catch (Exception e) {
			if (DEBUG) {
				System.out.println("Exception ignored");
				e.printStackTrace();
			}
		}
	}

	private void mainDialogShuttingDown() {
		if (DEBUG) {
			System.out.println("x=" + this.getX() + " y=" + this.getY());
			System.out.println("h/w = " + this.getHeight() + " / " + this.getWidth());
		}
		this.commandDispatcher.dispatch(CommandDispatcher.Command.SHUTTING_DOWN);
		this.clientReceiveParser.shutdown();
		properties.setMainX(getX());
		properties.setMainY(getY());
		properties.setMainWidth(getWidth());
		properties.setMainHeight(getHeight());
		properties.setMainTopBottomSplit(this.getTopBottomSplitPane().getDividerLocation());
		properties.setMainTopLRSplit(this.getTopLeftRightSplitPane().getDividerLocation());
		properties.setMainBotLRSplit(this.getBottomLeftRightSplitPane().getDividerLocation());
	}
	
	/** Return a reference to our board.
	 * This is the master board in the BuckoFIBS system.
	 * @return The Board
	 */
	public Board	getBoard() {
		return(board);
	}

	public void onMatchOver(String winner, String score) {
		this.board.onMatchOver(winner, score);
		updateBoard();
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());

			jContentPane.add(getToolbar(), BorderLayout.NORTH);
			jContentPane.add(getTopBottomSplitPane(), BorderLayout.CENTER);
		}
		return jContentPane;
	}

	/**
	 * This method initializes toolbar	
	 * 	
	 * @return javax.swing.JToolBar	
	 */
	private ToolBar getToolbar() {
		if (toolbar == null) {
			toolbar = new ToolBar();
			toolbar.setCommandDispatcher(commandDispatcher);
		}
		return toolbar;
	}

	/**
	 * This method initializes topBottomSplitPane	
	 * 	
	 * @return javax.swing.JSplitPane	
	 */
	private JSplitPane getTopBottomSplitPane() {
		if (topBottomSplitPane == null) {
			topBottomSplitPane = new JSplitPane();
			topBottomSplitPane.setOrientation(JSplitPane.VERTICAL_SPLIT);
			topBottomSplitPane.setTopComponent(getTopLeftRightSplitPane());
			topBottomSplitPane.setBottomComponent(getBottomLeftRightSplitPane());
			topBottomSplitPane.setDividerLocation(properties.getMainTopBottomSplit());
		}
		return topBottomSplitPane;
	}

	/**
	 * This method initializes topLeftRightSplitPane	
	 * 	
	 * @return javax.swing.JSplitPane	
	 */
	private JSplitPane getTopLeftRightSplitPane() {
		if (topLeftRightSplitPane == null) {
			topLeftRightSplitPane = new JSplitPane();
			topLeftRightSplitPane.setDividerLocation(properties.getMainTopLRSplit());
			topLeftRightSplitPane.setLeftComponent(getTopLeftTabbedPane());
			topLeftRightSplitPane.setRightComponent(getTopRightTabbedPane());
		}
		return topLeftRightSplitPane;
	}

	/**
	 * This method initializes bottomLeftRightSplitPane	
	 * 	
	 * @return javax.swing.JSplitPane	
	 */
	private JSplitPane getBottomLeftRightSplitPane() {
		if (bottomLeftRightSplitPane == null) {
			bottomLeftRightSplitPane = new JSplitPane();
			bottomLeftRightSplitPane.setDividerLocation(properties.getMainBotLRSplit());
			bottomLeftRightSplitPane.setRightComponent(getChatPane());
		}
		return bottomLeftRightSplitPane;
	}

	/**
	 * This method initializes topLeftTabbedPane	
	 * 	
	 * @return javax.swing.JTabbedPane	
	 */
	private JTabbedPane getTopLeftTabbedPane() {
		if (topLeftTabbedPane == null) {
			topLeftTabbedPane = new JTabbedPane();
			topLeftTabbedPane.addTab("Players", null, getPlayerListPane(), null);
			topLeftTabbedPane.addTab("Board", null, getBoardTab(), null);
		}
		return topLeftTabbedPane;
	}

	/**
	 * This method initializes playerListlPane	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	public PlayerListPane getPlayerListPane() {
		if (playerListPane == null) {
			playerListPane = new PlayerListPane();
			playerListPane.setCommandDispatcher(this.commandDispatcher);
		}
		return playerListPane;
	}

	private	ChatPane	getChatPane() {
		if (bottomRightChatPane == null) {
			bottomRightChatPane = new ChatPane();
			bottomRightChatPane.setCommandDispatcher(this.commandDispatcher);
		}
		return(bottomRightChatPane);
	}

	/** This method initializes topRightTabbedPane
	 * @return javax.swing.JTabbedPane	
	 */
	private JTabbedPane getTopRightTabbedPane() {
		if (topRightTabbedPane == null) {
			topRightTabbedPane = new JTabbedPane();
			topRightTabbedPane.addTab("System", null, getSystemMessagesScrollPane(), null);
			topRightTabbedPane.addTab("Invites", null, getInviterListPane(), null);
		}
		return topRightTabbedPane;
	}

	/**
	 * This method initializes systemMessagesTextPane	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	public JScrollPane getSystemMessagesScrollPane() {
		if (systemMessagesScrollPane == null) {
			systemMessagesTextPane = new SystemMessagesTextPane();
			systemMessagesScrollPane = new JScrollPane(systemMessagesTextPane);
			systemMessagesScrollPane.setAutoscrolls(true);
			systemMessagesDimension = new Dimension(10,10);
			systemMessagesRectangle = new Rectangle(10,10);
		}
		return systemMessagesScrollPane;
	}

	public SystemMessagesTextPane getSystemMessagesTextPane() {
		return(this.systemMessagesTextPane);
	}
	
	/**
	 * This method initializes systemMessagesTextPane	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	public JScrollPane getGameMessagesScrollPane() {
		if (gameMessagesScrollPane == null) {
			gameMessagesTextPane = new SystemMessagesTextPane();
			gameMessagesScrollPane = new JScrollPane(gameMessagesTextPane);
			gameMessagesScrollPane.setAutoscrolls(true);
			gameMessagesScrollPane.setVisible(false);
			gameMessagesDimension = new Dimension(10,10);
			gameMessagesRectangle = new Rectangle(10,10);
		}
		return gameMessagesScrollPane;
	}

	public void addChatMessage(String s) {
		bottomRightChatPane.writeChatMessage(s);
	}

	/**
	 * This method initializes systemMessagesTextPane	
	 * 	
	 * @return javax.swing.JTextPane	
	 */
	private JScrollPane getInviterListPane() {
		if (inviterListPane == null) {
			inviterListPane = new InviterListPane();
			inviterListPane.setCommandDispatcher(this.commandDispatcher);
			inviterListScrollPane = new JScrollPane(inviterListPane);
			inviterListScrollPane.setAutoscrolls(true);
		}
		return inviterListScrollPane;
	}

	private	BoardTab	getBoardTab() {
		if (boardTab == null) {
			boardTab = new BoardTab();
			boardTab.setBoard(board);
			boardTab.setCommandDispatcher(this.commandDispatcher);
		}
		return(boardTab);
	}

}
