/******************************************************************************
 * PreferencesDialog.java - Allow the user to manage his preferences 
 * $Id: PreferencesDialog.java,v 1.10 2009/02/14 13:16:26 dick Exp $
 * 
 * BuckoFIBS - Backgammon by BuckoSoft
 * Copyright 2008 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: PreferencesDialog.java,v $
 * Revision 1.10  2009/02/14 13:16:26  dick
 * BuckoFIBS is released under the GNU license.
 *
 * Revision 1.9  2009/02/12 06:26:45  dick
 * Add the audioCues property.
 *
 * Revision 1.8  2009/01/28 19:12:45  dick
 * Move all of the debug message settings to the messages preferences dialog.
 * Add hoverHelper.
 *
 * Revision 1.7  2009/01/18 06:17:18  dick
 * Get rid of DebugNotReady.  NotReady is a real user attribute.
 *
 * Revision 1.6  2009/01/18 04:57:27  dick
 * The owner is a JFrame.
 * Add showPointNumbers.
 *
 * Revision 1.5  2009/01/09 07:00:21  dick
 * stdoutNetMessages.
 *
 * Revision 1.4  2009/01/04 21:30:53  dick
 * Add allowMultiplePersonalities
 *
 * Revision 1.3  2008/12/14 06:44:13  dick
 * Handle showUnhandledCookies.
 *
 * Revision 1.2  2008/12/13 06:24:30  dick
 * Add NotReady and ShowUnhandledCookies.
 *
 * Revision 1.1  2008/12/11 08:49:05  dick
 * Allow the user to manage his preferences.
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is BuckoFIBS, <http://www.buckosoft.com/BuckoFIBS/>.
 * The Initial Developer of the Original Code is Dick Balaska and BuckoSoft, Corp.
 * 
 */
package com.buckosoft.fibs.BuckoFIBS.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.BevelBorder;

import com.buckosoft.fibs.BuckoFIBS.BFProperties;
import com.buckosoft.fibs.BuckoFIBS.CommandDispatcher;

/** Allow the user to manage his preferences.
 * @author Dick Balaska
 * @since 2008/12/11
 * @version $Revision: 1.10 $ <br> $Date: 2009/02/14 13:16:26 $
 * @see <a href="http://cvs.buckosoft.com/Projects/BuckoFIBS/BuckoFIBS/src/com/buckosoft/fibs/BuckoFIBS/gui/PreferencesDialog.java">cvs PreferencesDialog.java</a>
 */
public class PreferencesDialog extends JDialog {
	private static final long serialVersionUID = 1L;
	private	CommandDispatcher	commandDispatcher;  //  @jve:decl-index=0:
	private	BFProperties		properties;

	private JPanel jContentPane = null;
	private JButton jButtonOK = null;
	private JButton jButtonCancel = null;
	private JPanel jPanelOkCancel = null;
	private JPanel jPanelPrefs = null;
	private JLabel jLabelAutoConnect = null;
	private JCheckBox jCheckBoxAutoConnectOnStartup = null;
	private JLabel jLabelSettingsLabel = null;
	private JLabel jLabelMultiplePersonalities = null;
	private JCheckBox jCheckBoxAllowMultiplePersonalities = null;
	private JLabel jLabel1 = null;
	private JCheckBox jCheckBoxShowPointNumbers = null;
	private JLabel jLabel = null;
	private JCheckBox jCheckBoxHoverHelpers = null;
	private JLabel jLabel2 = null;
	private JCheckBox jCheckBoxAudioCues = null;

	/** no bean constructor allowed 
	 * 
	 */
	protected PreferencesDialog() {
		super();
		initialize();
	}

	/** Create and display the preferences dialog.
	 * Just set and forget.  It's modal and manages ok and cancel on its own.
	 * @param owner Probably an instance of MainDialog
	 * @param commandDispatcher The BuckoFIBS commandDispatcher
	 */
	public PreferencesDialog(JFrame owner, CommandDispatcher commandDispatcher) {
		super(owner, true);
		this.commandDispatcher = commandDispatcher;
		this.properties = commandDispatcher.getProperties();
		initialize();
		this.setLocationRelativeTo(owner);
		
		BFProperties props = this.commandDispatcher.getProperties();
		this.jCheckBoxAutoConnectOnStartup.getModel().setSelected(props.isAutoConnect());
		this.jCheckBoxShowPointNumbers.getModel().setSelected(props.isShowPointNumbers());
		this.jCheckBoxAllowMultiplePersonalities.getModel().setSelected(props.isAllowMultiplePersonalities());
		this.jCheckBoxHoverHelpers.getModel().setSelected(props.isHoverHelpers());
		this.jCheckBoxAudioCues.getModel().setSelected(props.isAudioCues());
	}

	private void okButtonActionPerformed(java.awt.event.ActionEvent evt) {
		setVisible(false);
		this.properties.setAutoConnect(this.jCheckBoxAutoConnectOnStartup.getModel().isSelected());
		this.properties.setShowPointNumbers(this.jCheckBoxShowPointNumbers.getModel().isSelected());
		this.properties.setAllowMultiplePersonalities(this.jCheckBoxAllowMultiplePersonalities.getModel().isSelected());
		this.properties.setHoverHelpers(this.jCheckBoxHoverHelpers.getModel().isSelected());
		this.properties.setAudioCues(this.jCheckBoxAudioCues.getModel().isSelected());
		dispose();
	}
	private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt) {
		setVisible(false);
		dispose();
	}

	/**
	 * This method initializes this
	 * 
	 */
	private void initialize() {
        this.setSize(new Dimension(408, 215));
        this.setContentPane(getJContentPane());
        this.setTitle("Preferences");
			
	}

	/**
	 * This method initializes jContentPane	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			GridBagConstraints gridBagConstraints6 = new GridBagConstraints();
			gridBagConstraints6.gridx = 0;
			gridBagConstraints6.weighty = 0.3;
			gridBagConstraints6.gridy = 0;
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.gridx = 0;
			gridBagConstraints2.anchor = GridBagConstraints.SOUTH;
			gridBagConstraints2.weighty = 0.2;
			gridBagConstraints2.gridwidth = 1;
			gridBagConstraints2.gridy = 2;
			jContentPane = new JPanel();
			jContentPane.setLayout(new GridBagLayout());
			jContentPane.add(getJPanelOkCancel(), gridBagConstraints2);
			jContentPane.add(getJPanelPrefs(), gridBagConstraints6);
		}
		return jContentPane;
	}

	/**
	 * This method initializes jButtonOK	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButtonOK() {
		if (jButtonOK == null) {
			jButtonOK = new JButton();
			jButtonOK.setText("OK");
			jButtonOK.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					okButtonActionPerformed(e);
				}
			});
		}
		return jButtonOK;
	}

	/**
	 * This method initializes jButtonCancel	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButtonCancel() {
		if (jButtonCancel == null) {
			jButtonCancel = new JButton();
			jButtonCancel.setText("Cancel");
			jButtonCancel.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					cancelButtonActionPerformed(e);
				}
			});
		}
		return jButtonCancel;
	}

	/**
	 * This method initializes jPanelOkCancel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanelOkCancel() {
		if (jPanelOkCancel == null) {
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.gridx = -1;
			gridBagConstraints1.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints1.insets = new Insets(0, 15, 0, 0);
			gridBagConstraints1.gridy = -1;
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.gridx = -1;
			gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints.anchor = GridBagConstraints.EAST;
			gridBagConstraints.weightx = 0.1;
			gridBagConstraints.insets = new Insets(0, 0, 0, 15);
			gridBagConstraints.gridy = -1;
			jPanelOkCancel = new JPanel();
			jPanelOkCancel.setLayout(new GridBagLayout());
			jPanelOkCancel.add(getJButtonOK(), gridBagConstraints);
			jPanelOkCancel.add(getJButtonCancel(), gridBagConstraints1);
		}
		return jPanelOkCancel;
	}

	/**
	 * This method initializes jPanelPrefs	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanelPrefs() {
		if (jPanelPrefs == null) {
			GridBagConstraints gridBagConstraints7 = new GridBagConstraints();
			gridBagConstraints7.gridx = 1;
			gridBagConstraints7.gridy = 2;
			GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
			gridBagConstraints5.gridx = 0;
			gridBagConstraints5.anchor = GridBagConstraints.WEST;
			gridBagConstraints5.gridy = 2;
			jLabel2 = new JLabel();
			jLabel2.setText("Make Noise:");
			GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
			gridBagConstraints4.gridx = 1;
			gridBagConstraints4.gridy = 5;
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
			gridBagConstraints3.gridx = 0;
			gridBagConstraints3.anchor = GridBagConstraints.WEST;
			gridBagConstraints3.gridy = 5;
			jLabel = new JLabel();
			jLabel.setText("Hover Helpers on the board:");
			GridBagConstraints gridBagConstraints22 = new GridBagConstraints();
			gridBagConstraints22.gridx = 1;
			gridBagConstraints22.gridy = 4;
			GridBagConstraints gridBagConstraints21 = new GridBagConstraints();
			gridBagConstraints21.gridx = 0;
			gridBagConstraints21.anchor = GridBagConstraints.WEST;
			gridBagConstraints21.insets = new Insets(0, 0, 0, 8);
			gridBagConstraints21.gridy = 4;
			jLabel1 = new JLabel();
			jLabel1.setText("Show Numbers on the points:");
			GridBagConstraints gridBagConstraints18 = new GridBagConstraints();
			gridBagConstraints18.gridx = 1;
			gridBagConstraints18.gridy = 3;
			GridBagConstraints gridBagConstraints17 = new GridBagConstraints();
			gridBagConstraints17.gridx = 0;
			gridBagConstraints17.anchor = GridBagConstraints.WEST;
			gridBagConstraints17.gridy = 3;
			jLabelMultiplePersonalities = new JLabel();
			jLabelMultiplePersonalities.setText("Allow Multiple Personalities:");
			GridBagConstraints gridBagConstraints12 = new GridBagConstraints();
			gridBagConstraints12.gridy = 1;
			GridBagConstraints gridBagConstraints11 = new GridBagConstraints();
			gridBagConstraints11.gridx = 0;
			gridBagConstraints11.anchor = GridBagConstraints.WEST;
			gridBagConstraints11.gridy = 0;
			jLabelSettingsLabel = new JLabel();
			jLabelSettingsLabel.setText("Settings:");
			jLabelSettingsLabel.setForeground(new Color(102, 102, 0));
			jLabelSettingsLabel.setFont(new Font("Dialog", Font.BOLD, 14));
			GridBagConstraints gridBagConstraints10 = new GridBagConstraints();
			gridBagConstraints10.anchor = GridBagConstraints.WEST;
			gridBagConstraints10.gridy = 1;
			gridBagConstraints10.weightx = 0.2;
			jLabelAutoConnect = new JLabel();
			jLabelAutoConnect.setText("Auto Connect on Startup:");
			jPanelPrefs = new JPanel();
			jPanelPrefs.setLayout(new GridBagLayout());
			jPanelPrefs.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
			jPanelPrefs.add(jLabelAutoConnect, gridBagConstraints10);
			jPanelPrefs.add(getJCheckBoxAutoConnectOnStartup(), gridBagConstraints12);
			jPanelPrefs.add(jLabelSettingsLabel, gridBagConstraints11);
			jPanelPrefs.add(jLabelMultiplePersonalities, gridBagConstraints17);
			jPanelPrefs.add(getJCheckBoxAllowMultiplePersonalities(), gridBagConstraints18);
			jPanelPrefs.add(jLabel1, gridBagConstraints21);
			jPanelPrefs.add(getJCheckBoxShowPointNumbers(), gridBagConstraints22);
			jPanelPrefs.add(jLabel, gridBagConstraints3);
			jPanelPrefs.add(getJCheckBoxHoverHelpers(), gridBagConstraints4);
			jPanelPrefs.add(jLabel2, gridBagConstraints5);
			jPanelPrefs.add(getJCheckBoxAudioCues(), gridBagConstraints7);
		}
		return jPanelPrefs;
	}

	/**
	 * This method initializes jCheckBoxAutoConnectOnStartup	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */
	private JCheckBox getJCheckBoxAutoConnectOnStartup() {
		if (jCheckBoxAutoConnectOnStartup == null) {
			jCheckBoxAutoConnectOnStartup = new JCheckBox();
		}
		return jCheckBoxAutoConnectOnStartup;
	}

	/**
	 * This method initializes jCheckBoxAllowMultiplePersonalities	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */
	private JCheckBox getJCheckBoxAllowMultiplePersonalities() {
		if (jCheckBoxAllowMultiplePersonalities == null) {
			jCheckBoxAllowMultiplePersonalities = new JCheckBox();
		}
		return jCheckBoxAllowMultiplePersonalities;
	}

	/**
	 * This method initializes jCheckBoxShowPointNumbers	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */
	private JCheckBox getJCheckBoxShowPointNumbers() {
		if (jCheckBoxShowPointNumbers == null) {
			jCheckBoxShowPointNumbers = new JCheckBox();
		}
		return jCheckBoxShowPointNumbers;
	}

	/**
	 * This method initializes jCheckBoxHoverHelpers	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */
	private JCheckBox getJCheckBoxHoverHelpers() {
		if (jCheckBoxHoverHelpers == null) {
			jCheckBoxHoverHelpers = new JCheckBox();
		}
		return jCheckBoxHoverHelpers;
	}

	/**
	 * This method initializes jCheckBoxAudioCues	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */
	private JCheckBox getJCheckBoxAudioCues() {
		if (jCheckBoxAudioCues == null) {
			jCheckBoxAudioCues = new JCheckBox();
		}
		return jCheckBoxAudioCues;
	}

}  //  @jve:decl-index=0:visual-constraint="10,10"
