/******************************************************************************
 * FIBSMessages.java - Define the possible message types from FIBS. 
 * $Id: FIBSMessages.java,v 1.13 2009/02/14 12:30:21 dick Exp $
 * 
 * BuckoFIBS - Backgammon by BuckoSoft
 * Copyright 2008 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: FIBSMessages.java,v $
 * Revision 1.13  2009/02/14 12:30:21  dick
 * BuckoFIBS is released under the GNU license.
 *
 * Revision 1.12  2009/02/11 09:09:43  dick
 * Better beginning and ending your turn.
 *
 * Revision 1.11  2009/02/09 22:46:21  dick
 * Add EchoJunk, a message we understand, but don't care about and should be echoed to the system pane.
 *
 * Revision 1.10  2009/02/06 07:56:48  dick
 * Working on doubles.
 *
 * Revision 1.9  2009/02/05 05:57:23  dick
 * Rework the sequence because i had 2 #43s (oops).
 *
 * Revision 1.8  2009/02/02 08:37:11  dick
 * Add FIBS_RollBeforeMove().
 *
 * Revision 1.7  2009/01/31 06:04:19  dick
 * Add YouAlreadyPlaying.
 *
 * Revision 1.6  2009/01/28 08:25:42  dick
 * Add HasSavedGames and Talk to Self.
 *
 * Revision 1.5  2009/01/10 05:11:04  dick
 * Add FIBS_PointsFor, a resuming match message.
 *
 * Revision 1.4  2009/01/09 07:33:22  dick
 * Add FIBS_NotInteresting.
 *
 * Revision 1.3  2009/01/07 04:29:16  dick
 * Javadoc.
 *
 * Revision 1.2  2009/01/06 08:05:28  dick
 * Handle registering a new user.
 *
 * Revision 1.1  2008/12/13 06:57:11  dick
 * Move ClientConnection, CookieMonster, and FIBSMessages to their own network package.
 *
 * Revision 1.3  2008/12/11 10:00:00  dick
 * Add FIBS_BoardstyleSetTo.
 *
 * Revision 1.2  2008/12/09 20:02:44  dick
 * Javadoc cleanup.
 *
 * Revision 1.1  2008/10/10 09:02:30  dick
 * Finish implementing the FIBS CookieMonster.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is BuckoFIBS, <http://www.buckosoft.com/BuckoFIBS/>.
 * The Initial Developer of the Original Code is Dick Balaska and BuckoSoft, Corp.
 * 
 */
package com.buckosoft.fibs.net;

/** Define the possible message types from FIBS.
 * @author cppc
 * @author Dick Balaska
 * @author Original author of C version says not to use his name
 * @since 2008/10/08
 * @version $Revision: 1.13 $ <br> $Date: 2009/02/14 12:30:21 $
 * @see <a href="http://www.fibs.com/fcm/">http://www.fibs.com/fcm/</a> <br>
 * @see <a href="http://cvs.buckosoft.com/Projects/BuckoFIBS/BuckoFIBS/src/com/buckosoft/fibs/net/FIBSMessages.java">cvs FIBSMessages.java</a>
 */
public interface FIBSMessages {

	final static int	CLIP_VERSION	= 1008;

	final static int	CLIP_WELCOME			= 1;
	final static int	CLIP_OWN_INFO			= 2;
	final static int	CLIP_MOTD_BEGIN			= 3;
	final static int	CLIP_MOTD_END			= 4;
	final static int	CLIP_WHO_INFO			= 5;
	final static int	CLIP_WHO_END			= 6;
	final static int	CLIP_LOGIN				= 7;
	final static int	CLIP_LOGOUT				= 8;
	final static int	CLIP_MESSAGE			= 9;
	final static int	CLIP_MESSAGE_DELIVERED	= 10;
	final static int	CLIP_MESSAGE_SAVED		= 11;
	final static int	CLIP_SAYS				= 12;
	final static int	CLIP_SHOUTS				= 13;
	final static int	CLIP_WHISPERS			= 14;
	final static int	CLIP_KIBITZES			= 15;
	final static int	CLIP_YOU_SAY			= 16;
	final static int	CLIP_YOU_SHOUT			= 17;
	final static int	CLIP_YOU_WHISPER		= 18;
	final static int	CLIP_YOU_KIBITZ			= 19;
	
	final static int	FIBS_BAD_COOKIE			= -1;
	final static int	FIBS_PreLogin			= 20;
	final static int	FIBS_LoginPrompt		= 21;
	final static int	FIBS_FailedLogin		= 22;
	final static int	FIBS_MOTD				= 23;
	final static int	FIBS_Goodbye			= 24;
	final static int	FIBS_PostGoodbye		= 25;	// "send cookies", etc.
	final static int	FIBS_Unknown			= 26;	// don't know the type, probably can ignore
	final static int	FIBS_Empty				= 27;	// empty string
	final static int	FIBS_Junk				= 28;	// a message we don't care about, but is not unknown
	final static int	FIBS_ClearScreen		= 29;
	final static int	FIBS_BAD_AcceptDouble	= 30;	// DANGER, WILL ROBINSON!!! See notes in .c file about these two cookies!
	final static int	FIBS_BAD_Board			= 31;
	final static int	FIBS_Average			= 32;
	final static int	FIBS_DiceTest			= 33;
	final static int	FIBS_Stat				= 34;
	final static int	FIBS_Why				= 35;
	final static int	FIBS_NoInfo				= 36;
	final static int	FIBS_LastLogout			= 37;
	final static int	FIBS_RatingCalcStart	= 38;
	final static int	FIBS_RatingCalcInfo		= 39;
	final static int	FIBS_SettingsHeader		= 40;
	final static int	FIBS_PlayerListHeader	= 41;
	final static int	FIBS_AwayListHeader		= 42;
	final static int	FIBS_RatingExperience	= 43;

	final static int	FIBS_NotLoggedIn		= 44;
	final static int	FIBS_StillLoggedIn		= 45;
	final static int	FIBS_NoOneIsAway		= 46;
	final static int	FIBS_RatingsHeader		= 47;
	final static int	FIBS_IsPlayingWith		= 48;
	final static int	FIBS_Timeout			= 49;
	final static int	FIBS_UnknownCommand		= 50;
	final static int	FIBS_Username			= 51;
	final static int	FIBS_LastLogin			= 52;
	final static int	FIBS_YourLastLogin		= 53;
	final static int	FIBS_Registered			= 54;
	final static int	FIBS_ONEUSERNAME		= 55;
	final static int	FIBS_EnterUsername		= 56;
	final static int	FIBS_EnterPassword		= 57;
	final static int	FIBS_TypeInNo			= 58;
	final static int	FIBS_SavedScoreHeader	= 59;
	final static int	FIBS_NoSavedGames		= 60;
	final static int	FIBS_UsersHeardYou		= 61;
	final static int	FIBS_MessagesForYou		= 62;
	final static int	FIBS_IsAway				= 63;
	final static int	FIBS_OpponentLogsOut	= 64;
	final static int	FIBS_Waves				= 65;
	final static int	FIBS_WavesAgain			= 66;
	final static int	FIBS_YouGag				= 67;
	final static int	FIBS_YouUngag			= 68;
	final static int	FIBS_YouBlind			= 69;
	final static int	FIBS_YouUnblind			= 70;
	final static int	FIBS_WatchResign		= 71;
	final static int	FIBS_UseToggleReady		= 72;
	final static int	FIBS_WARNINGSavedMatch	= 73;
	final static int	FIBS_NoSavedMatch		= 74;
	final static int	FIBS_AlreadyPlaying		= 75;
	final static int	FIBS_DidntInvite		= 76;
	final static int	FIBS_WatchingHeader		= 77;
	final static int	FIBS_NotWatching		= 78;
	final static int	FIBS_NotWatchingPlaying = 79;
	final static int	FIBS_NotPlaying			= 80;
	final static int	FIBS_PlayerNotPlaying	= 81;
	final static int	FIBS_NoUser				= 82;
	final static int	FIBS_CantInviteSelf		= 83;
	final static int	FIBS_CantWatch			= 84;
	final static int	FIBS_CantTalk			= 85;
	final static int	FIBS_CantBlindYourself	= 86;
	final static int	FIBS_WontListen			= 87;
	final static int	FIBS_TypeBack			= 88;
	final static int	FIBS_NoOne				= 89;
	final static int	FIBS_BadMove			= 90;
	final static int	FIBS_MustMove			= 91;
	final static int	FIBS_MustComeIn			= 92;
	final static int	FIBS_CantShout			= 93;
	final static int	FIBS_DontKnowUser		= 94;
	final static int	FIBS_MessageUsage		= 95;
	final static int	FIBS_Done				= 96;
	final static int	FIBS_SavedMatchesHeader	= 97;
	final static int	FIBS_CantGagYourself	= 98;
	final static int	FIBS_NotYourTurnToRoll	= 99;
	final static int	FIBS_NotYourTurnToMove	= 100;
	final static int	FIBS_YourTurnToMove		= 101;
	final static int	FIBS_Ratings			= 102;
	final static int	FIBS_PlayerInfoStart	= 103;
	final static int	FIBS_EmailAddress		= 104;
	final static int	FIBS_NoEmail			= 105;
	final static int	FIBS_ListOfGames		= 106;
	final static int	FIBS_SavedMatch			= 107;
	final static int	FIBS_SavedMatchPlaying	= 108;
	final static int	FIBS_SavedMatchReady	= 109;
	final static int	FIBS_YouAreWatching		= 110;
	final static int	FIBS_YouStopWatching	= 111;
	final static int	FIBS_PlayerStartsWatching	= 112;
	final static int	FIBS_PlayerStopsWatching	= 113;
	final static int	FIBS_PlayerIsWatching		= 114;
	final static int	FIBS_ReportUnlimitedMatch	= 115;
	final static int	FIBS_ReportLimitedMatch	= 116;
	final static int	FIBS_RollOrDouble		= 117;
	final static int	FIBS_RollBeforeMove		= 118;
	final static int	FIBS_YouWinMatch		= 119;
	final static int	FIBS_PlayerWinsMatch	= 120;
	final static int	FIBS_YouReject			= 121;
	final static int	FIBS_YouResign			= 122;
	final static int	FIBS_ResumeMatchRequest	= 123;
	final static int	FIBS_ResumeMatchAck0	= 124;
	final static int	FIBS_ResumeMatchAck5	= 125;
	final static int	FIBS_NewMatchRequest	= 126;
	final static int	FIBS_UnlimitedInvite	= 127;
	final static int	FIBS_YouInvited			= 128;
	final static int	FIBS_YouAlreadyPlaying	= 129;
	final static int	FIBS_NewMatchAck9		= 130;
	final static int	FIBS_NewMatchAck10		= 131;
	final static int	FIBS_NewMatchAck2		= 132;
	final static int	FIBS_YouTerminated		= 133;
	final static int	FIBS_OpponentLeftGame	= 134;
	final static int	FIBS_PlayerLeftGame		= 135;
	final static int	FIBS_PlayerRefusingGames= 136;
	final static int	FIBS_TypeJoin			= 137;
	final static int	FIBS_ShowMovesStart		= 138;
	final static int	FIBS_ShowMovesWins		= 139;
	final static int	FIBS_ShowMovesRoll		= 140;
	final static int	FIBS_ShowMovesDoubles	= 141;
	final static int	FIBS_ShowMovesAccepts	= 142;
	final static int	FIBS_ShowMovesRejects	= 143;
	final static int	FIBS_ShowMovesOther		= 144;
	final static int	FIBS_Board				= 145;
	final static int	FIBS_YouRoll			= 146;
	final static int	FIBS_PlayerRolls		= 147;
	final static int	FIBS_PlayerMoves		= 148;
	final static int	FIBS_Doubles			= 149;
	final static int	FIBS_AcceptRejectDouble	= 150;
	final static int	FIBS_StartingNewGame	= 151;
	final static int	FIBS_PlayerAcceptsDouble= 152;
	final static int	FIBS_YouAcceptDouble	= 153;
	final static int	FIBS_Settings			= 154;
	final static int	FIBS_Turn				= 155;
	final static int	FIBS_FirstRoll			= 156;
	final static int	FIBS_DoublingCubeNow	= 157;
	final static int	FIBS_CantMove			= 158;
	final static int	FIBS_CantMoveFirstMove	= 159;
	final static int	FIBS_ResignRefused		= 160;
	final static int	FIBS_YouWinGame			= 161;
	final static int	FIBS_OnlyPossibleMove	= 162;
	final static int	FIBS_AcceptWins			= 163;
	final static int	FIBS_ResignWins			= 164;
	final static int	FIBS_ResignYouWin		= 165;
	final static int	FIBS_WatchGameWins		= 166;
	final static int	FIBS_ScoreUpdate		= 167;
	final static int	FIBS_MatchStart			= 168;
	final static int	FIBS_YouAcceptAndWin	= 169;
	final static int	FIBS_OnlyMove			= 170;
	final static int	FIBS_BearingOff			= 171;
	final static int	FIBS_PleaseMove			= 172;
	final static int	FIBS_MakesFirstMove		= 173;
	final static int	FIBS_YouDouble			= 174;
	final static int	FIBS_MatchLength		= 175;
	final static int	FIBS_PlayerWantsToResign= 176;
	final static int	FIBS_PlayerWinsGame		= 177;
	final static int	FIBS_JoinNextGame		= 178;
	final static int	FIBS_ResumingUnlimitedMatch	= 179;
	final static int	FIBS_ResumingLimitedMatch	= 180;
	final static int	FIBS_PlayersStartingMatch	= 181;
	final static int	FIBS_PlayersStartingUnlimitedMatch	= 182;
	final static int	FIBS_MatchResult		= 183;
	final static int	FIBS_YouGiveUp			= 184;
	final static int	FIBS_PlayerIsWaitingForYou	= 185;
	final static int	FIBS_Boardstyle			= 186;
	final static int	FIBS_Linelength			= 187;
	final static int	FIBS_Pagelength			= 188;
	final static int	FIBS_Redoubles			= 189;
	final static int	FIBS_Sortwho			= 190;
	final static int	FIBS_Timezone			= 191;
	final static int	FIBS_RedoublesSetTo		= 192;
	final static int	FIBS_BoardstyleSetTo	= 193;
	final static int	FIBS_AllowpipTrue		= 194;
	final static int	FIBS_AllowpipFalse		= 195;
	final static int	FIBS_AutoboardTrue		= 196;
	final static int	FIBS_AutoboardFalse		= 197;
	final static int	FIBS_AutodoubleTrue		= 198;
	final static int	FIBS_AutodoubleFalse	= 199;
	final static int	FIBS_AutomoveTrue		= 200;
	final static int	FIBS_AutomoveFalse		= 201;
	final static int	FIBS_BellTrue			= 202;
	final static int	FIBS_BellFalse			= 203;
	final static int	FIBS_CrawfordTrue		= 204;
	final static int	FIBS_CrawfordFalse		= 205;
	final static int	FIBS_DoubleTrue			= 206;
	final static int	FIBS_DoubleFalse		= 207;
	final static int	FIBS_MoreboardsTrue		= 208;
	final static int	FIBS_MoreboardsFalse	= 209;
	final static int	FIBS_MovesTrue			= 210;
	final static int	FIBS_MovesFalse			= 211;
	final static int	FIBS_GreedyTrue			= 212;
	final static int	FIBS_GreedyFalse		= 213;
	final static int	FIBS_NotifyTrue			= 214;
	final static int	FIBS_NotifyFalse		= 215;
	final static int	FIBS_RatingsTrue		= 216;
	final static int	FIBS_RatingsFalse		= 217;
	final static int	FIBS_ReadyTrue			= 218;
	final static int	FIBS_ReadyFalse			= 219;
	final static int	FIBS_ReportTrue			= 220;
	final static int	FIBS_ReportFalse		= 221;
	final static int	FIBS_SilentTrue			= 222;
	final static int	FIBS_SilentFalse		= 223;
	final static int	FIBS_TelnetTrue			= 224;
	final static int	FIBS_TelnetFalse		= 225;
	final static int	FIBS_WrapTrue			= 226;
	final static int	FIBS_WrapFalse			= 227;
	final static int	FIBS_WelcomeToFibs		= 228;
	final static int	FIBS_OneUserPerPerson	= 229;
	final static int	FIBS_UseAnotherName		= 230;
	final static int	FIBS_GivePassword		= 231;
	final static int	FIBS_RetypePassword		= 232;
	final static int	FIBS_YouAreRegistered	= 233;
	final static int	FIBS_NotInteresting		= 234;
	final static int	FIBS_PointsFor			= 235;	// during a resume match
	final static int	FIBS_HasSavedGames		= 236;	// during a resume match
	final static int	FIBS_TalkToSelf			= 237;	// during a resume match
	final static int	FIBS_ReadyWatchingPlaying=238;
	final static int	FIBS_PleaseWaitForJoin	= 239;
	final static int	FIBS_YouAlreadyRolled	= 240;
	final static int	FIBS_YouCantMove		= 241;
	
	final static int	FIBS_EchoJunk			= 242;	// just echo this to the System window
	

		
	//final static int	FIBS_LastMessage		= 235;	// NO MORE MESSAGES HERE!
}

